package im.zego.metaworlddemo.utils;

import android.util.Log;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.liulishuo.okdownload.DownloadTask;
import com.liulishuo.okdownload.SpeedCalculator;
import com.liulishuo.okdownload.core.breakpoint.BlockInfo;
import com.liulishuo.okdownload.core.breakpoint.BreakpointInfo;
import com.liulishuo.okdownload.core.cause.EndCause;
import com.liulishuo.okdownload.core.dispatcher.DownloadDispatcher;
import com.liulishuo.okdownload.core.listener.DownloadListener4WithSpeed;
import com.liulishuo.okdownload.core.listener.assist.Listener4SpeedAssistExtend;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import im.zego.metaworld.ZMWBasePackageLoader;
import im.zego.metaworlddemo.view.MWApplication;

public class OKPackageDownloader extends ZMWBasePackageLoader {
    private static final String TAG = "MWOKPackageDownloader";
    private static final String OSS_BASE_PATH = "https://zego-public.oss-cn-shanghai.aliyuncs.com/solution_source/release/home/2.0.2/Assets";
    private static final boolean AUTO_CHECK_UPDATE = true;
    private CombineTaskDownloader mInnerDownloader = new CombineTaskDownloader();
    private Set<String> mCheckedUpdatedPackageID = new HashSet<>();

    private static OKPackageDownloader sInstance;


    public static OKPackageDownloader getInstance() {
        if (sInstance == null) {
            sInstance = new OKPackageDownloader();
        }
        return sInstance;
    }

    private OKPackageDownloader() {
    }

    @Override
    public void active() {
        super.active();
        DownloadDispatcher.setMaxParallelRunningCount(3); //最大并行下载数
    }

    @Override
    public void onRequiredPackage(String packageID, PackageLoadCallBack callBack) {
        Log.d(TAG, "OnRequiredPackage: " + packageID);

        String packageParentPath = getPackageParentPath();
        checkNeedUpdated(packageID, (needUpdated, remoteHash) -> {
            Log.d(TAG, "Check need updated: " + needUpdated + ", id: " + packageID + ", remoteHash: " + remoteHash);
            if (!needUpdated) {
                Log.d(TAG, "Dont need to update package: " + packageID);
                callBack.onResult(0, packageParentPath + "/" + packageID);
            } else {
                startDownloadPackage(packageID, packageParentPath, remoteHash, callBack);
            }
        });
    }

    /**
     * 获取 package 存储的父目录
     */
    private String getPackageParentPath() {
        return MWApplication.application.getExternalFilesDir(null).getAbsolutePath() + "/MetaAssets/AAAssets";
    }

    /**
     * 获取 package hash 文件
     */
    private String getPackageHashPath(String packageID) {
        return getPackageParentPath() + "/" + packageID + "/" + packageID + ".hash";
    }

    /**
     * 开始下载 package
     */
    private void startDownloadPackage(String packageID, String packageParentPath, String remoteHash, PackageLoadCallBack callBack) {
        String packageURL;
        packageURL = OSS_BASE_PATH + "/" + packageID + "/Android/" + packageID + ".zip";
        mInnerDownloader.download(packageID, packageURL, getPackageParentPath(), true, new CombineTaskDownloadListener() {
            @Override
            public void onProgress(float progress) {
                callBack.onProgress(progress);
            }

            @Override
            public void onResult(int errorCode, File downloadedFile) {
                if (errorCode == 0) {
                    // 把 hash 信息写入本地 hash 文件
                    writeFileContent(getPackageHashPath(packageID), remoteHash);
                    mCheckedUpdatedPackageID.add(packageID);
                }
                callBack.onResult(errorCode, packageParentPath + "/" + packageID);
            }
        });
    }

    /**
     * 检查 package 是否需要更新
     */
    private void checkNeedUpdated(String packageID, NeedUpdatedListener cb) {
        if (!AUTO_CHECK_UPDATE) {
            cb.onResult(false, null);
            return;
        }
        if (mCheckedUpdatedPackageID.contains(packageID)) {
            // 之前已经检查过更新，不需要重复检查
            Log.d(TAG, "Cache hit, id: " + packageID);
            cb.onResult(false, null);
            return;
        }

        String localHashFilePath = getPackageHashPath(packageID);
        String localHashContent = readFileContent(localHashFilePath);
        String hashURL;
        hashURL = OSS_BASE_PATH + "/" + packageID + "/Android/" + packageID + ".hash";
        final String finalLocalContent = localHashContent;
        mInnerDownloader.download(packageID + "-hash", hashURL, getPackageParentPath(), true, new CombineTaskDownloadListener() {
            @Override
            public void onProgress(float progress) {
            }

            @Override
            public void onResult(int errorCode, File downloadedFile) {
                if (errorCode != 0) {
                    // 远端 hash 下载失败，不需要更新
                    Log.w(TAG, "Download hash error, id: " + packageID + ", code: " + errorCode);
                    cb.onResult(false, null);
                } else {
                    String remoteHashContent = readFileContent(downloadedFile.getAbsolutePath());
                    boolean needUpdated = remoteHashContent != null && !remoteHashContent.equals(finalLocalContent);
                    Log.d(TAG, "Download hash success, needUpdated: " + needUpdated + ", localHash: " + finalLocalContent + ", remoteHash: " + remoteHashContent);
                    cb.onResult(remoteHashContent != null && !remoteHashContent.equals(finalLocalContent), remoteHashContent);
                }
            }
        });
    }

    private String readFileContent(String filePath) {
        File file = new File(filePath);
        if (!file.exists()) {
            return null;
        }
        InputStream is = null;
        try {
            is = new FileInputStream(file);
            InputStreamReader isr = new InputStreamReader(is, "UTF-8");
            BufferedReader bfr = new BufferedReader(isr);
            String instring = "";
            StringBuilder stringBuilder = new StringBuilder();
            while ((instring = bfr.readLine()) != null) {
                stringBuilder.append(instring);
            }
            return stringBuilder.toString();
        } catch (Exception e) {
            Log.w(TAG, "Read local file error: " + e.getMessage());
            return null;
        } finally {
            if (is != null) {
                try {
                    is.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }

    private void writeFileContent(String filePath, String content) {
        File file = new File(filePath);
        if (file.exists()) {
            file.delete();
        }
        OutputStream os = null;
        try {
            os = new FileOutputStream(file);
            os.write(content.getBytes(StandardCharsets.UTF_8));
        } catch (Exception e) {
            Log.w(TAG, "Write local file error: " + e.getMessage());
        } finally {
            if (os != null) {
                try {
                    os.flush();
                    os.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }

    /**
     * 带有任务合并的下载器
     * 1. 直接用 OKDownload 并行相同的下载任务会失败
     * 2. OKDownload 回调太复杂了，做接口简化
     */
    private static class CombineTaskDownloader {
        private static final String TAG = "CombineTaskDownloader";
        private Map<String, List<CombineTaskDownloadListener>> mPendingDownloadListeners = new HashMap<>();

        public void download(String taskID, String url, String localParentPath, boolean autoUnZip, CombineTaskDownloadListener listener) {
            Log.d(TAG, "Start download, task id: " + taskID + ", url: " + url + ", parentPath: " + localParentPath);
            synchronized (CombineTaskDownloader.class) {
                if (mPendingDownloadListeners.containsKey(taskID)) {
                    // 已经有 job 了，不用重复发起任务
                    Log.d(TAG, "OnRequiredPackage append job");
                    List<CombineTaskDownloadListener> listeners = mPendingDownloadListeners.get(taskID);
                    listeners.add(listener);
                    return;
                }
                List<CombineTaskDownloadListener> listeners = new ArrayList<>();
                listeners.add(listener);
                mPendingDownloadListeners.put(taskID, listeners);
            }

            File parentDirectory = new File(localParentPath);
            if (!parentDirectory.exists()) {
                parentDirectory.mkdirs();
            }

            DownloadTask task = new DownloadTask.Builder(url, parentDirectory)
                    .setMinIntervalMillisCallbackProcess(30) // 下载进度回调的间隔时间（毫秒）
                    .setPassIfAlreadyCompleted(true)// 任务过去已完成是否要重新下载
                    .setAutoCallbackToUIThread(true)
                    .setConnectionCount(1)
                    .build();

            task.enqueue(new MWDownloadListener4WithSpeed(taskID, new CombineTaskDownloadListener() {
                @Override
                public void onProgress(float progress) {
                    synchronized (PackageLoadCallBack.class) {
                        List<CombineTaskDownloadListener> listeners = mPendingDownloadListeners.get(taskID);
                        if (listeners == null) {
                            return;
                        }
                        for (CombineTaskDownloadListener l : listeners) {
                            l.onProgress(progress);
                        }
                    }
                }

                @Override
                public void onResult(int errorCode, File downloadedFile) {
                    final String zipSuffix = ".zip";
                    if (errorCode == 0 && autoUnZip && downloadedFile.getPath().endsWith(zipSuffix)) {
                        try {
                            ZipUtils.unzipFile(downloadedFile.getAbsolutePath(), downloadedFile.getParent() + "/" + downloadedFile.getName().replaceAll(zipSuffix, ""), true);
                        } catch (IOException e) {
                            Log.w(TAG, "Unzip error: " + e.getMessage());
                            errorCode = -3;
                        }
                    }
                    synchronized (PackageLoadCallBack.class) {
                        List<CombineTaskDownloadListener> listeners = mPendingDownloadListeners.get(taskID);
                        if (listeners == null) {
                            return;
                        }
                        for (CombineTaskDownloadListener l : listeners) {
                            l.onResult(errorCode, downloadedFile);
                        }
                        mPendingDownloadListeners.remove(taskID);
                    }
                }
            }));
        }
    }

    interface CombineTaskDownloadListener {
        void onProgress(float progress);

        void onResult(int errorCode, File downloadedFile);
    }

    /**
     * 下载任务监听器
     */
    private static class MWDownloadListener4WithSpeed extends DownloadListener4WithSpeed {
        private long totalLength;
        private final CombineTaskDownloadListener outCallBack;
        private final String taskID;

        public MWDownloadListener4WithSpeed(String taskID, CombineTaskDownloadListener callBack) {
            this.taskID = taskID;
            this.outCallBack = callBack;
        }

        @Override
        public void taskStart(@NonNull DownloadTask task) {
            Log.i(TAG, "DownloadListener start task, id: " + taskID);
        }

        @Override
        public void infoReady(@NonNull DownloadTask task, @NonNull BreakpointInfo info,
                              boolean fromBreakpoint, @NonNull Listener4SpeedAssistExtend.Listener4SpeedModel model) {
            totalLength = info.getTotalLength();
        }

        @Override
        public void connectStart(@NonNull DownloadTask task, int blockIndex,
                                 @NonNull Map<String, List<String>> requestHeaders) {
        }

        @Override
        public void connectEnd(@NonNull DownloadTask task, int blockIndex, int responseCode,
                               @NonNull Map<String, List<String>> responseHeaders) {
            Log.i(TAG, "DownloadListener connectEnd, id: " + taskID + ", response: " + responseCode);
        }

        @Override
        public void progressBlock(@NonNull DownloadTask task, int blockIndex,
                                  long currentBlockOffset, @NonNull SpeedCalculator blockSpeed) {
        }

        @Override
        public void progress(@NonNull DownloadTask task, long currentOffset,
                             @NonNull SpeedCalculator taskSpeed) {
            float percent = (float) currentOffset / totalLength;
            outCallBack.onProgress(percent);
        }

        @Override
        public void blockEnd(@NonNull DownloadTask task, int blockIndex, BlockInfo
                info, @NonNull SpeedCalculator blockSpeed) {
        }

        @Override
        public void taskEnd(@NonNull DownloadTask task, @NonNull EndCause
                cause, @Nullable Exception realCause, @NonNull SpeedCalculator taskSpeed) {
            Log.i(TAG, "DownloadListener taskEnd, id: " + taskID + ", cause: " + cause + ", exception: " + realCause + ", speed: " + taskSpeed.averageSpeed());
            outCallBack.onResult(cause == EndCause.COMPLETED ? 0 : -1, task.getFile());
        }
    }

    // 是否需要更新资源回调
    public interface NeedUpdatedListener {
        void onResult(boolean needUpdated, String remoteHash);
    }
}
